// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//

import document from "document";
import { vibration } from "haptics";
import clock from "clock";
import display from "display";
import { memory } from "system";
import { HeartRateSensor } from "heart-rate";
import { today } from 'user-activity';
import { charger, battery } from "power";
import { logger } from "../common/utils";
import * as utils from "../common/utils";
import * as names from "../common/names";
import tz_data from "../common/tz";


let V;
let P;
let T;
let C;

let scr_width;
let scr_height;
let cur_dom = -1;
var aod_on = false;

var heart_rate = "--";

let fire = 0;

var bpm_showing = true;
let hrm;
const days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]

export function init(_V)
{
  let bar;

  V = _V;
  C = V.G.C;
  P = V.G.P;
  T = V.G.T;

  utils.scr_init("Face", V);
  
  adj_width(document.getElementById("face"));

  //
  // Watch face init
  //
  utils.image_btn("lr_btn", "icons/moon-0.png", (btn) => {   
    if (T.running)
      timer_set_lap();
    else {
      utils.bounce("config1", V);
    }
  });

  for (let i = 0; i < 12; i++) {
    bar = document.getElementById("tick-" + i);
    bar.groupTransform.rotate.angle = (i * 30);
  }

  let stats_click = document.getElementById("stats_click");
  stats_click.addEventListener("click", (evt) => {
    bpm_showing = !bpm_showing;
    document.getElementById("stats_img").image = (bpm_showing ? "icons/bpm.png" :
                                                              "icons/steps.png");
    sh_stats();
  })

  let time2_click = document.getElementById("time2_click");
  time2_click.addEventListener("click", (evt) => {
    utils.bounce("timer", V);
  })
  timer_show(false);

  hrm = new HeartRateSensor();
  hrm.onreading = function() {
    if (hrm.heartRate) {
      heart_rate = hrm.heartRate;
      if (heart_rate == 0)
        heart_rate = "--";
    }
    hrm.stop();
  }

  set_rotation();
  set_moon("icons/moon-0.png");
  //
  // Flashlight init
  //

  //
  // tap on center of screen toggles flashlight
  //
  let bg = document.getElementById("click_bg");
  bg.addEventListener("click", (evt) => {
    turn_on(true);
  })
  //
  // tap on battery of screen toggles flashlight
  //
  let bat = document.getElementById("battery_click");
  bat.addEventListener("click", (evt) => {
    turn_on(false);
  })

  let flash = document.getElementById("click_flash");
  flash.addEventListener("click", (evt) => {
    turn_off();
  })

  //
  // tap on day of month brings up calendar
  //
  let mo = document.getElementById("date_touch");
  mo.layer = 2;
  mo.addEventListener("click", (evt) => {
    mo.style.fill = "cyan";
    mo.style.opacity = 0.5;
    utils.bounce("calendar_%", V, { date: V.G.ev_date });
  });

  //
  // tap on day of week brings up future events
  //
  let wk = document.getElementById("week_touch");
  wk.layer = 2;
  wk.addEventListener("click", (evt) => {
    utils.bounce("event", V, {
                                days: true
                             });
  });

  hi_event(new Date());
  timer_show(T.enable);

  updateClock();
  clock.granularity = "seconds";
  clock.ontick = () => updateClock();

  return () => {
    // Unsubscribe from clock.
    clock.granularity = "off";
    clock.ontick = void 0;
    hrm.stop();
  }

  return false;
}

export function aod(on)
{
logger("aod(" + on + ")");
return;
  aod_on = on;
  let screen = document.getElementById("analog_face");
  if (on) {
    screen.getElementById("secs").style.display = "none";
    screen.getElementById("face_timer").style.display = "none";
    screen.getElementById("stats").style.display = "none";
    screen.getElementById("stats_icon").style.display = "none";
    screen.getElementById("battery").style.display = "none";
    screen.getElementById("lr_btn").style.display = "none";
    hrm.stop();
  } else {
    screen.getElementById("secs").style.display = "inline";
    screen.getElementById("face_timer").style.display = "inline";
    screen.getElementById("stats").style.display = "inline";
    screen.getElementById("stats_icon").style.display = "inline";
    screen.getElementById("battery").style.display = "inline";
    screen.getElementById("lr_btn").style.display = "inline";
    hrm.start();
  }
}

//
//  Watch face functions
//

//
//  The Ionic is the only Fitbit device with a non-square
//  screen (it's wider than it is tall).  Since all of the UI
//  elements are proportional to the height of the screen
//  this means that elements in the middle row of the
//  face (the '9', day of week, day of month and '3') would
//  be wrong on the Ionic.  This routine adjusts the elements
//  properly (do the adjustment for all devices, it winds up
//  being a no-op for square screens).
//
function adj_width(r)
{
  var e;

  scr_width = r.width;
  scr_height = r.height;
//logger("Screen: " + r.width + "x" + r.height);
  if (r.width == r.height)
    return;

  let c1 = r.width / 2;
  let c2 = r.height / 2;
  e = document.getElementById("min-3");
  e.x = (c1 + c2) - 42;
  e = document.getElementById("date_box");
  e.x = (c1 + c2) - 96;
  e = document.getElementById("min-9");
  e.x = (c1 - c2) + 42;
  e = document.getElementById("wk_box");
  e.x = (c1 - c2) + 50;
}

function haptic_intr()
{

  vibration.stop();
  haptic = 0;
}

function do_haptic(state, time)
{


  time = parseInt(time / 1000);
  if (state && ((time % 10) == 0)) {
    let type = "alert";
    vibration.start(type);
    haptic = setTimeout(haptic_intr, 2000);
  }
}

function timer_state()
{
  let tz;

  let e = utils.elapsed(Date.now(), T);
  if (T.running) {
    if (T.mode == C.TIMER_UP) {
      tz = "lap " + (T.lapn + 1);
    } else {
      if (e <= 0) {
        tz = "elapsed";
        do_haptic(T.running, e);
      } else {
        tz = "down";
      }
    }
  } else if (T.mode == C.TIMER_DOWN)
    tz = (e == T.itime) ? "timer" : "stopped";
  else
    tz = (e == 0) ? "chrono" : "pause";
  let time = utils.milli2str(e, false);
  return { tz, time };
}

function timer_set_lap()
{

  utils.laps_set(Date.now() - T.stime, T);
  T.laps[T.lapn] = 0;
}

function timer_intr()
{

  sched_intr(1000, Date.now() - T.stime);
}

export function sched_intr(delay, ticks)
{

  if (fire != 0) {
    clearTimeout(fire);
    fire = 0;
  }
  if (T.running) {
    let t = parseInt(ticks / 1000);
    ticks = ticks - (t * 1000);
    fire = setTimeout(timer_intr, delay - ticks);
  }
}

function timer_start()
{

  T.laps[T.lapn] = 0;
  var ticks = T.etime - T.stime;
  T.running = true;
  T.stime = Date.now() - ticks;
  if (T.mode == C.TIMER_UP)
    set_moon("icons/lap.png");
  //sched_intr(1000, ticks);
}

function timer_stop()
{

  T.etime = Date.now();
  T.laps[T.lapn] = T.etime - T.stime;
  T.running = false;
  set_moon(null);
}

function timer_ss()
{

  if (T.running)
    timer_stop();
  else
    timer_start();
}

export function timer_show(enable)
{

  if (enable) {
    document.getElementById("time2").style.display = "inline";
    document.getElementById("time2_icon").style.display = "none";
  } else {
    document.getElementById("time2").style.display = (P.tz2 == C.TZ_NONE2) ? "none" : "inline";
    document.getElementById("time2_icon").style.display = (P.tz2 == C.TZ_NONE2) ? "inline" : "none";
  }
}

export function set_rotation()
{

  let min3 = document.getElementById("min-3");
  let min9 = document.getElementById("min-9");
  if (P.rotation < 0) {
    min3.text = "9";
    min9.text = "3";
  } else {
    min3.text = "3";
    min9.text = "9";
  }
}

function moon_image(local)
{

  if (local == null)
    local = new Date();
  cur_dom = local.getDate();
  return utils.moon_phase(local);
}

export function set_moon(image)
{

  if (image == null)
    image = moon_image(null);
  let btn = document.getElementById("lr_btn");
  let lr_icon = btn.getElementById("b_image");
  lr_icon.image = image;
}

function hi_event(date)
{

  let bg = document.getElementById("date_bg");
  if (bg != undefined) {
    let busy = utils.day_events(date.getFullYear(), date.getMonth(), date.getDate());
    bg.style.fill = busy ? "darkslategray" : "black";
  }
}

export function sh_moon(local)
{

  if (local.getDate() != cur_dom) {
    set_moon(moon_image(local));
    hi_event(local);
  }
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  let hourAngle = (360 / 12) * hours;
  let minAngle = (360 / 12 / 60) * minutes;
  return (hourAngle + minAngle) * P.rotation;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) {
  return ((360 / 60) * minutes) * P.rotation;
}

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) {
  return ((360 / 60) * seconds) * P.rotation;
}

function sh_battery() {
  
  let lvl = Math.floor(battery.chargeLevel);
  let batValue = document.getElementById("battery_text");
  batValue.text = lvl;
  if (lvl > 50)
    batValue.style.fill = "lime";
  else if (lvl > 25)
    batValue.style.fill = "yellow";
  else
    batValue.style.fill = "red";
}

function find_tz(idx, utc)
{

  if (utils.dst(idx, utc))
    return tz_data[idx].daylight;
  return tz_data[idx].standard;
}

function tz_info(idx)
{

  if (idx == C.TZ_CUST1_IDX)
	  return { standard: "TZ1", off: P.custom_tz1 };
  else if (idx == C.TZ_CUST2_IDX)
	  return { standard: "TZ2", off: P.custom_tz2 };
  else
    return tz_data[idx];
}

function get_time2(tz2_idx, utc_time, tm2)
{
  var tz, time;
  
  if (T.enable) {
    return timer_state();
  } else if (tz2_idx >= 0) {
    tz = find_tz(tz2_idx, utc_time);
    let hr = tm2.getHours();
    let ampm = utils.ampm_mark(hr, P, " A", " P");
    if (!P.hr24 && (hr > 12))
      hr -= 12;
    let mn = tm2.getMinutes();
    time = utils.numPad(hr, " ") + ":" + utils.numPad(mn, "0");
    if (!P.hr24)
      time += ampm;
  } else {
    tz = "";
    time = "";
  }
  return { tz, time };
}

function sh_tz2(tz2_idx, tm2, utc_time)
{

  let { tz, time } = get_time2(tz2_idx, utc_time, tm2);
  let time2 = document.getElementById("time2");
  time2.getElementById("tz2").text = tz;
  utils.text_bck(time, time2);
}

function tz_off(idx, utc)
{

  let info = tz_info(idx);
  let off = info.off;
  if (utils.dst(info, utc))
    off += 60;
  return off;
}

function sh_stats()
{

  let txt = bpm_showing ? heart_rate : utils.comma(today.adjusted.steps);
  utils.text_bck(txt, document.getElementById("stats"));
}

// Rotate the hands every tick
export function updateClock()
{

  let local = new Date();
//logger("updateClock - " + local.getSeconds());
  let tm1 = local;
  let tm2;
  let tz1_idx = names.get_info(P.tz1, C).tz;
  let tz2_idx = names.get_info(P.tz2, C).tz;
  if (tz1_idx > 0 || tz2_idx >= 0) {
    let off = utils.utc_off(tm1.toString());
    let utc_time = utils.utc(tm1, -off);
    if (tz1_idx > 0)
      tm1 = utils.utc(utc_time, tz_off(tz1_idx, utc_time));
    if (tz2_idx > 0)
      tm2 = utils.utc(utc_time, tz_off(tz2_idx, utc_time));
    else if (tz2_idx == 0)
      tm2 = local;
  }
  let hours = tm1.getHours();
  let mins = tm1.getMinutes();
  let secs = tm1.getSeconds();

  document.getElementById("pm").text = utils.ampm_mark(hours, P, "AM", "PM");

  let hourHand = document.getElementById("hours");
  hourHand.groupTransform.rotate.angle = hoursToAngle(hours%12, mins);

  let minHand = document.getElementById("mins");
  minHand.groupTransform.rotate.angle = minutesToAngle(mins);

  let secHand = document.getElementById("secs");
  secHand.groupTransform.rotate.angle = secondsToAngle(secs);

  document.getElementById("tz1").text = ((tz1_idx == 0) ? "" : find_tz(tz1_idx, utc_time));
  document.getElementById("date").text = tm1.getDate();
  document.getElementById("week").text = days[tm1.getDay()];
  
  // upper left corner
  sh_battery();
  
  // upper right corner
  if (!aod_on)
    hrm.start();
  sh_stats();

  // lower left corner
  sh_tz2(tz2_idx, tm2, utc_time);

  // lower right corner
  sh_moon(local);
}

//
//  Flashlight functions
//

function show(watch, flash)
{

  document.getElementById("watch_face").style.display = watch;
  document.getElementById("flashlight").style.display = flash;
}

function turn_on(center)
{

  if (center && T.enable) {
    timer_ss();
  } else {
    show("none", "inline");
    display.autoOff = false;
    display.brightnessOverride = "max";
    V.G.flashlight = true;
  }
}

function turn_off()
{

  show("inline", "none");
  display.autoOff = true;
  display.brightnessOverride = undefined;
  V.G.flashlight = false;
}
