// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//
import document from "document";
import { memory } from "system";
import * as fs from "fs";

let log_enable = 1;

let ev_date;
let ev_days;

var m_names = [
  "January",  "February", "March",
  "April",    "May",      "June",
  "July",     "August",   "September",
  "October",  "November", "December"
];

export function logger(str)
{

  if (log_enable)
    console.log("DDD: " + str);
}

export function sh_mem(str)
{

  logger(str + ": " + memory.js.used + "/" + memory.js.total);
}

export function pr_obj(title, object, details) {

  logger(title);
  for(let key in object) {
    if (details) {
      try {
       logger('  ' + key + ':' + JSON.stringify(object[key]));
      } catch (error) {
        // Some values throw an error when trying to access them.
        logger('  ' + key + '|' + error.message);
     }
    } else {
      logger('  ' + key + "-");
    }
  }
}

/*
 * List the contents of the file system area
 *
 * Sigh.  Fitbit has decided that directories are too much
 * trouble so we only have a flat file system with multiple
 * files.
 */
export function ls()
{
  let dirIter, listDir;

  logger("/private/data:");
  try {
    listDir = fs.listDirSync("/private/data");
  } catch (error) {
    logger("ls:listDirSync failed - " + error);
  }
  while((dirIter = listDir.next()) && !dirIter.done) {
    logger("  " + dirIter.value);
  }
}

// Add zero in front of numbers < 10
export function numPad(i, p) {
  if (i < 10) {
    i = p + i;
  }
  return "" + i;
}

export function add_event(index, year, month, day, e)
{
  let evts;

  let ev = day_events(year, month, day);
  if (ev) {
    let del = 1;
    if (index < 0) {
      del = 0;
      for (index = 0; index < ev.length; index++)
        if (ev[index].start > e.start)
          break;
    }
    ev.splice(index, del, e);
  } else {
    ev = [ e ];
  }
  ev_days[day] = ev;
  let fn = "C-" + year + "-" + numPad(month, "0");
  try {
    fs.writeFileSync(fn, ev_days, "json");
  } catch (error) {
    logger("addEvent:writeFileSync failed - " + error);
  }
}

export function rm_event(index, year, month, day)
{

  let ev = day_events(year, month, day);
  if (!ev)
    return;
  if (index >= ev.length)
    return;
  ev.splice(index, 1);
  if (ev.length == 0)
    ev = undefined;
  ev_days[day] = ev;
  let fn = "C-" + year + "-" + numPad(month, "0");
  try {
    fs.writeFileSync(fn, ev_days, "json");
  } catch (error) {
    logger("rm_event:writeFileSync failed - " + error);
  }
}

export function cmp_dates(m1, m2)
{

  if (m1 === m2)
    return 0;
  return (m1 < m2) ? -1 : 1;
}

function ev_file(fn)
{

  return fn.substr(0, 2) === "C-";
}

function clear_month(month, old)
{

  if (!ev_file(month))
    return false;
  if (old)
    return cmp_dates(month, old) < 0;
  return true;
}

export function clear_events(old)
{
  let dirIter, listDir;

  if (old) {
    let date = new Date();
    old = "C-" + date.getFullYear() + "-" + numPad(date.getMonth(), "0");
  }
  try {
    listDir = fs.listDirSync("/private/data");
  } catch (error) {
    logger("clear_events:listDirSync failed - " + error);
  }
  while((dirIter = listDir.next()) && !dirIter.done) {
    if (clear_month(dirIter.value, old))
      try {
        fs.unlinkSync(dirIter.value);
      } catch (error) {
        logger("clear_events:unlinkSync failed - " + error);
      }
  }
}

export function list_events(d)
{
  let dirIter, listDir;

  let events = [];
  let idx = 0;
  try {
    listDir = fs.listDirSync("/private/data");
  } catch (error) {
    logger("list_events:listDirSync failed - " + error);
  }
  while((dirIter = listDir.next()) && !dirIter.done)
    if (ev_file(dirIter.value))
      events[idx++] = dirIter.value;
  events.sort();
  let old = 0;

  if (d) {
    let comp = "C-" + d.getFullYear() + "-" + numPad(d.getMonth(), "0");
    let num_events = events.length;
    for (old = 0; old < num_events; ++old)
      if (cmp_dates(events[old], comp) >= 0)
        break;
  }
  return {
            old: old,
            events: events
         };
}

export function get_days(year, month)
{
  let events;

  let fn = "C-" + year + "-" + numPad(month, "0");
  try {
    return fs.readFileSync(fn, "json");
  } catch (error) {
    return undefined;
  }
}

export function day_events(year, month, day)
{
  let events;

  if (day < 1)
    return;

  ev_date = new Date(year, month, day);
  ev_days = [];

  let fn = "C-" + year + "-" + numPad(month, "0");
  try {
    ev_days = fs.readFileSync(fn, "json");
  } catch (error) {
    return undefined;
  }
  return ev_days[day];
}

export function laps_set(t, T)
{

  T.laps[T.lapn++] = t - T.laps_ticks;
  T.laps_ticks = t;
  return T.lapn;
}

export function laps_reset(T)
{

  T.lapn = 0;
  T.laps = [];
  T.laps[0] = 0;
  T.laps_ticks = 0;
}

export function elapsed(ticks, T)
{

  if (T.running)
    ticks -= T.stime;
  else
    ticks = T.etime - T.stime;
  return (T.mode == 0) ? ticks : (T.itime - ticks);
}

export function image_btn(id, image, action)
{

  let btn = document.getElementById(id);
  if (btn) {
    let img = btn.getElementById("b_image");
    img.image = image;
    let touch = btn.getElementById("b_touch");
    touch.addEventListener("click", (evt) => {
      action(btn);
    })
  }
}

export function text_btn(id, title, c_fore, c_back, action)
{

  let btn = document.getElementById(id);
  if (btn) {
    if (c_back) {
      let back = btn.getElementById("b_back");
      back.style.fill = c_back;
    }
    let fore = btn.getElementById("b_text");
    if (c_fore)
      fore.style.fill = c_fore;
    if (title)
      fore.text = title;
    let touch = btn.getElementById("b_touch");
    touch.addEventListener("click", (evt) => {
      action(btn);
    })
  }
}

export function show_btn(name, visible)
{

  let btn = document.getElementById(name);
  btn.style.display = visible ? "inline" : "none";
}

export function label_btn(name, title, fill)
{

  let t = document.getElementById(name).getElementById("b_text");
  t.text = title;
  if (fill)
    t.style.fill = fill;
}

// Add zero in front of numbeers < 10
export function zeroPad(i) {
  if (i < 10)
    i = "0" + i;
  return i;
}

export function cell_idx(ev, x, y, w, h)
{

  let sx = ev.screenX;
  let sy = ev.screenY;

  r = parseInt((sy - y) / h) * 7;
  return r + parseInt((sx - x) / w);
}

export function bounce(name, _V, opt)
{

  if (_V.G.sdk === "sdk4")
    _V.replace("bounce", {
                            name: name,
                            opt: opt
                         });
  else
    _V.replace(name, opt);
}

export function scr_init(title, views)
{

  text_btn("um_btn", title, "fb-white", "fb-blue", (btn) => {
    bounce("face_%", views);
  });
}

export function utc(time, off)
{

  let yr = time.getFullYear();
  let mo = time.getMonth();
  let day = time.getDate();
  let hr = time.getHours();
  let mn = time.getMinutes();
  let sec = time.getSeconds();

  let d = 0;
  let min = ((hr * 60) + mn) + off;
  if (min < 0) {
    d = -1;
    min += (24 * 60);
  } else if (min >= (24 * 60)) {
    d = 1;
    min -= (24 * 60);
  }
  hr = parseInt(min / 60);
  min -= (hr * 60);
  return new Date(yr, mo, day, hr, min, sec);
}

export function utc_off(gmt)
{
  let sign;

  gmt = gmt.substring(gmt.indexOf("GMT") + 3);
  let sign = gmt.charAt(0);
  let idx = gmt.indexOf(":");
  let h = parseInt(gmt.substring(1, idx));
  let m = parseInt(gmt.substring(idx + 1));
  let r = (h * 60) + (m * 1);
  if (sign == "-")
    r = -r;
  return r;
}

export function m_name(month)
{

  return m_names[month];
}

export const mo_days = [  31,  28,  31,  30,  31,  30,  31,  31,  30,  31,  30,  31 ];
const mo_cumm = [   0,  31,  59,  90, 120, 151, 181, 212, 243, 273, 304, 334, 365 ];
function find_dom(day, month, year, begin)
{

  let time = new Date(year, month);   // month is 1 based
  let first = time.getDay();          // 0 based
  let off = day - first;
  if (off < 0)
    off += 7;
  let dom = 1 + off;
  if (begin)
    return dom;
  while ((dom + 7) <= mo_days[month])
    dom += 7;
  return dom;
}


function mod(num, denom)
{

  let d = parseInt(num / denom);
  return num - (d * denom);
}

function leap(yr, mo)
{

  if (mo != 1)
    return 0;
  if (mod(yr, 4) != 0)
    return 0;
  if (mod(yr, 100) != 0)
    return 1;
  return mod(yr, 400) ? 0 : 1;
}

//
// Number of days in a month (including leap year for Feb)
//
// year = 0 based
// monty = 0 based
//
export function days_per_mo(year, month)
{

  return mo_days[month] + leap(year, month);
}

//  Starts at 2AM on the 2nd Sunday in March
//  Ends at 2AM on the 1st Sunday in November
//
function dst_usa(time)
{

  let month = time.getMonth();
  if (month < 2 || month > 10)
    return false;
  if (month != 2 && month != 10)
    return true;

  let first = find_dom(0, month, time.getFullYear(), true);
  let off = (month == 2) ? 7 : 0;
  let anchor = new Date(time.getFullYear(), month, first + off, 2, 0, 0);
  if (month == 2) {
    return (time.getTime() < anchor.getTime()) ? false : true;
  } else {
    return (time.getTime() < anchor.getTime()) ? true : false;
  }
}

//  Starts at 1AM UTC on the last Sunday in March
//  Ends at 1AM UTC on the last Sunday in October
//
function dst_eu(time)
{

  let month = time.getMonth();
  if (month < 2 || month > 9)
    return false;
  if (month != 2 && month != 9)
    return true;

  let last = find_dom(0, month, time.getFullYear(), false);
  let tstr = time.getFullYear() + "-" + dig2(month + 1, "0") + "-" + dig2(last, "0") + "T01:00:00Z";
  let anchor = new Date(tstr);
  if (month == 2) {
    return (time.getTime() < anchor.getTime()) ? false : true;
  } else {
    return (time.getTime() < anchor.getTime()) ? true : false;
  }
}

export function dst(tz, time)
{
  let mo_day;

  if (tz.days == 0)
    return dst_usa(time);
  if (tz.days == 1)
    return dst_eu(time);
  return false;
}

export function dig2(num, lead)
{

  return (num < 10) ? (lead + num) : num;
}

export function min2str(off, sign, p)
{
  let hr, min, m;

  let ampm = "";
  let s="";
  if (off < 0) {
    s = "-";
    off = -off;
  } else if ((off > 0) && sign)
    s = "+";

  if (p && !p.hr24) {
    ampm = "A";
    if (off >= (12 * 60)) {
      off -= 12*60;
      ampm = "P";
    }
  }

  hr = parseInt(off / 60);
  min = off - (hr * 60);
  if (min < 0)
    min = -min;
  m = dig2(min, "0");
  
  if (p && !p.hr24 && hr == 0)
    hr = 12;
  
  return s + hr + ":" + m + ampm;
}

export function milli2str(milli, hundred)
{
  let sign;

  if (milli < 0) {
    sign = "-";
    milli = -milli;
  } else
    sign = "";
  milli = parseInt(milli / 10);
  let sec = parseInt(milli/100);
  let hun = milli - (sec * 100);
  let min = parseInt(sec / 60);
  sec = sec - (min * 60);
  let hr = parseInt(min / 60);
  min -= hr * 60;
  let ret = "";
  if (hundred) {
    ret = "." + numPad(hun, "0");
    if (hr == 0 && min == 0)
      return sign + sec + ret;
  }
  ret = numPad(sec, "0") + ret;
  if (hr == 0)
    return sign + min + "." + ret;
  return sign + hr + ":" + numPad(min, "0") + "." + ret;
}

export function ampm_mark(h, p, am_str, pm_str)
{

  if (p.hr24)
    return "";
  switch (p.ampm) {

  case 0:   // cfg_ampm = 0, cfg_am = 0
  case 2:   // cfg_ampm = 0, cfg_am = 1
    return "";

  case 3:   // cfg_ampm = 1, cfg_am = 0
    return (h >= 12) ? pm_str : "";

  case 1:   // cfg_ampm = 1, cfg_am = 1
    return (h >= 12) ? pm_str : am_str;


  }
  return "?";
}

export function get_ampm(w)
{

  let btn = w.getElementById("cfg_ampm0");
  if (btn.value != 0)
    return 0;
  btn = w.getElementById("cfg_ampm1");
  if (btn.value != 0)
    return 1;
  btn = w.getElementById("cfg_ampm2");
  if (btn.value != 0)
    return 2;
}

export function prefs_pr(title, P)
{

  console.log(title + " =>");
  console.log("  tz1: " + P.tz1);
  console.log("  tz2: " + P.tz2);
  console.log("  custom_tz1: " + P.custom_tz1);
  console.log("  custom_tz1: " + P.custom_tz1);
  console.log("  rotation: " + P.rotation);
  console.log("  ampm: " + P.ampm);
  console.log("  hr24: " + P.hr24);
}

export function prefs_save(P)
{
  let json = {
    tz1: P.tz1,
    tz2: P.tz2,
    rotation: P.rotation,
    custom_tz1: P.custom_tz1,
    custom_tz2: P.custom_tz2,
    ampm: P.ampm,
    hr24: P.hr24
  };

  try {
    fs.writeFileSync("prefs.txt", json, "json");
  } catch(e) {
    console.log("fs write error - " + e);
  }
}

export function epoch_days(d)
{

  let yr = d.getFullYear();
  if (d.getMonth() < 2)
    yr = yr - 1;
  let leaps = parseInt(yr/4) - parseInt(yr/100) + parseInt(yr/400);

  let days = (d.getFullYear() * 365) + d.getDate() + mo_cumm[d.getMonth()];
  return days + leaps;
}

export function moon_day(d)
{

  return parseInt((d + 1) % 29.530588853);
}

export function moon_icon(mday)
{

  mday = mday + 2;
  let d = moon_day(mday);
  let cycle = (moon_day(mday + 29) == d) ? 29 : 30;
  return parseInt((d * 8) / cycle);
}

export function lunar_day(d)
{

  let ndays = epoch_days(d) - epoch_days(new Date(1900, 0, 1, 0, 0, 0));
  return moon_day(ndays);
}

export function moon_phase(d)
{

  let ndays = epoch_days(d) - epoch_days(new Date(1900, 0, 1, 0, 0, 0));
  return "icons/moon-" + moon_icon(ndays) + ".png";
}

export function tz_split(name)
{

  let str = name.split(";");
  return { name: str[0], tz: parseInt(str[1]) };
}

export function comma(n)
{

  if (n < 1000)
    return "" + n;
  let u = parseInt(n / 1000);
  n = n - (u * 1000);
  var sep = ",";
  if (n < 100)
    sep += "0";
  if (n < 10)
    sep += "0";
  return u + sep + n;
}

//  Text with background
//
//  txt - text to display
//  t - text UI element for txt
//  b - background rectangle for the text
//  r - x coord of right side of text (optional)
//
//  Stretch the background box to cover the width of the
//  text.  If r is specified then the text is right
//  justified and r is the x coord of the right hand side.
//
export function text_bck(txt, e)
{

  let t = e.getElementById("info");
  t.text = txt;
  let bbox = t.getBBox();

  let b = e.getElementById("back");
  b.x = bbox.x;
  b.width = bbox.width;
}

export function mk_ascii(idx, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16)
{

  var r = "";
  let l = [ a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16 ];
  for (let i = 0; i < 16; i++) {
    if (!l[i])
      break;
    r = r + l[i];
  }
  r = r + ";" + idx;
  return r;
}
